@@ -3,10 +3,10 @@ App::DBBrowser::Join_Union;
 
 use warnings;
 use strict;
-use 5.008003;
+use 5.010000;
 no warnings 'utf8';
 
-our $VERSION = '0.049_09';
+our $VERSION = '0.041';
 
 use Clone                  qw( clone );
 use List::MoreUtils        qw( any );
@@ -18,6 +18,8 @@ use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use App::DBBrowser::DB;
 
+sub CLEAR_SCREEN () { "\e[H\e[J" }
+
 
 
 sub new {
@@ -26,13 +28,14 @@ sub new {
 }
 
 
+
 sub __union_tables {
     my ( $self, $dbh, $db, $schema, $data ) = @_;
     my $no_lyt = Term::Choose->new();
     my $u = $data->{$db}{$schema};
     if ( ! defined $u->{col_names} || ! defined $u->{col_types} ) {
         my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-        ( $u->{col_names}, $u->{col_types} ) = $obj_db->column_names_and_types( $dbh, $db, $schema, $u->{tables} );
+        $data = $obj_db->column_names_and_types( $dbh, $db, $schema, $data );
     }
     my $union = {
         unused_tables => [ map { "- $_" } @{$u->{tables}} ],
@@ -48,18 +51,17 @@ sub __union_tables {
         my @post_tbl = ( $all_tables, $self->{info}{_info} );
         my $choices = [ @pre_tbl, map( "+ $_", @{$union->{used_tables}} ), @{$union->{unused_tables}}, @post_tbl ];
         $self->__print_union_statement( $union );
-        my $prompt = $self->{union_all} ? 'One UNION table for cols:' : 'Choose UNION table:';
         # Choose
         my $idx_tbl = $no_lyt->choose(
             $choices,
-            { %{$self->{info}{lyt_stmt_v}}, prompt => $prompt, index => 1 }
+            { %{$self->{info}{lyt_stmt_v}}, prompt => 'Choose UNION table:', index => 1 }
         );
         return if ! defined $idx_tbl;
         my $union_table = $choices->[$idx_tbl];
         return  if ! defined $union_table;
         if ( $union_table eq $self->{info}{_info} ) {
             if ( ! defined $u->{tables_info} ) {
-                $u->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $u );
+                $u->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $data );
             }
             my $tbls_info = $self->__print_tables_info( $u );
             # Choose
@@ -185,8 +187,7 @@ sub __union_tables {
         $union->{quote}{stmt} .= " AS " . $dbh->quote_identifier( 'UNION_ALL_TABLES' );
     }
     else {
-        #$union->{quote}{stmt} .= " AS " . $dbh->quote_identifier( join '_', @{$union->{used_tables}} );
-        $union->{quote}{stmt} .= " AS " . $dbh->quote_identifier( 'UNION_SELECTED_TABLES' );
+        $union->{quote}{stmt} .= " AS " . $dbh->quote_identifier( join '_', @{$union->{used_tables}} );
     }
     return $union;
 }
@@ -196,79 +197,63 @@ sub __print_union_statement {
     my ( $self, $union ) = @_;
     my $str;
     if ( $self->{union_all} ) {
-        $str = 'UNION ALL TABLES';
+        $str = 'UNION ALL TABLES' . "\n" . 'Col: ';
         if ( @{$union->{used_tables}} ) {
-            $str .= "\n" . 'Cols: ';
             my $table = $union->{used_tables}[0];
-            if ( defined $union->{used_cols}{$table} && @{$union->{used_cols}{$table}} ) { #
-                $str .= join( ', ', @{$union->{used_cols}{$table}} );
-            }
+            $str .= join( ', ', @{$union->{used_cols}{$table}} ) if @{$union->{used_cols}{$table}//[]};
         }
         $str .= "\n";
     }
     else {
         $str = "SELECT * FROM (\n";
+        my $c = 0;
+        for my $table ( @{$union->{used_tables}} ) {
+            ++$c;
+            $str .= "  SELECT ";
+            $str .= @{$union->{used_cols}{$table}//[]} ? join( ', ', @{$union->{used_cols}{$table}} ) : '?';
+            $str .= " FROM $table";
+            $str .= $c < @{$union->{used_tables}} ? " UNION ALL\n" : "\n";
+        }
         if ( @{$union->{used_tables}} ) {
-            my $c = 0;
-            for my $table ( @{$union->{used_tables}} ) {
-                ++$c;
-                $str .= "  SELECT ";
-                if ( defined $union->{used_cols}{$table} && @{$union->{used_cols}{$table}} ) { #
-                    $str .= join( ', ', @{$union->{used_cols}{$table}} );
-                }
-                else {
-                    $str .= '?';
-                }
-                $str .= " FROM $table";
-                $str .= $c < @{$union->{used_tables}} ? " UNION ALL\n" : "\n";
-            }
             $str .= ") AS ";
-            #$str .= join '_', @{$union->{used_tables}};
-            $str .= 'Selected_Tables';
+            $str .= $self->{union_all} ? 'UNION_ALL_TABLES' : join '_', @{$union->{used_tables}};
             $str .= " \n";
         }
     }
     $str .= "\n";
     my $line_fold = Text::LineFold->new( %{$self->{info}{line_fold}}, ColMax => ( term_size() )[0] - 2 );
-    print $self->{info}{clear_screen};
+    print CLEAR_SCREEN;
     print $line_fold->fold( '', ' ' x $self->{info}{stmt_init_tab}, $str );
 }
 
 
 sub __get_tables_info {
-    my ( $self, $dbh, $db, $schema, $u_or_j ) = @_;
+    my ( $self, $dbh, $db, $schema, $data ) = @_;
     my $tables_info = {};
     my $sth;
     my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-    my ( $pk, $fk ) = $obj_db->primary_and_foreign_keys( $dbh, $db, $schema, $u_or_j->{tables} );
-    for my $table ( @{$u_or_j->{tables}} ) {
+    my ( $pk, $fk ) = $obj_db->primary_and_foreign_keys( $dbh, $db, $schema, $data );
+    for my $table ( @{$data->{$db}{$schema}{tables}} ) {
         push @{$tables_info->{$table}}, [ 'Table: ', '== ' . $table . ' ==' ];
-        if ( defined $u_or_j->{col_names} ) {
-            push @{$tables_info->{$table}}, [
-                'Columns: ',
-                join( ' | ', map {
-                        lc( $u_or_j->{col_types}{$table}[$_] )
-                    . ' ' . $u_or_j->{col_names}{$table}[$_] } 0 .. $#{$u_or_j->{col_names}{$table}} )
-            ];
-        }
-        if ( defined $pk && @{$pk->{$table}} ) {
+        push @{$tables_info->{$table}}, [
+            'Columns: ',
+            join( ' | ', map {
+                    lc( $data->{$db}{$schema}{col_types}{$table}[$_] )
+                . ' ' . $data->{$db}{$schema}{col_names}{$table}[$_] } 0 .. $#{$data->{$db}{$schema}{col_names}{$table}} )
+        ];
+        if ( @{$pk->{$table}} ) {
             push @{$tables_info->{$table}}, [ 'PK: ', 'primary key (' . join( ',', @{$pk->{$table}} ) . ')' ];
         }
-        if ( defined $fk ) {
-            for my $fk_name ( sort keys %{$fk->{$table}} ) {
-                if ( $fk->{$table}{$fk_name} ) {
-                    push @{$tables_info->{$table}}, [
-                        'FK: ',
-                        'foreign key (' . join( ',', @{$fk->{$table}{$fk_name}{foreign_key_col}} ) .
-                        ') references ' . $fk->{$table}{$fk_name}{reference_table} .
-                        '(' . join( ',', @{$fk->{$table}{$fk_name}{reference_key_col}} ) .')'
-                    ];
-                }
+        for my $fk_name ( sort keys %{$fk->{$table}} ) {
+            if ( $fk->{$table}{$fk_name} ) {
+                push @{$tables_info->{$table}}, [
+                    'FK: ',
+                    'foreign key (' . join( ',', @{$fk->{$table}{$fk_name}{foreign_key_col}} ) .
+                    ') references ' . $fk->{$table}{$fk_name}{reference_table} .
+                    '(' . join( ',', @{$fk->{$table}{$fk_name}{reference_key_col}} ) .')'
+                ];
             }
         }
-        if ( @{$tables_info->{$table}} == 1 ) {
-            push @{$tables_info->{$table}}, [ '   No INFO available.' ];
-        }
     }
     return $tables_info;
 }
@@ -302,7 +287,7 @@ sub __join_tables {
     my $j = $data->{$db}{$schema};
     if ( ! defined $j->{col_names} || ! defined $j->{col_types} ) {
         my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-        ( $j->{col_names}, $j->{col_types} ) = $obj_db->column_names_and_types( $dbh, $db, $schema, $j->{tables} );
+        $data = $obj_db->column_names_and_types( $dbh, $db, $schema, $data );
     }
     my @tables = map { "- $_" } @{$j->{tables}};
 
@@ -320,7 +305,7 @@ sub __join_tables {
         return if ! defined $master;
         if ( $master eq $self->{info}{_info} ) {
             if ( ! defined $j->{tables_info} ) {
-                $j->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $j );
+                $j->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $data );
             }
             my $tbls_info = $self->__print_tables_info( $j );
             # Choose
@@ -376,7 +361,7 @@ sub __join_tables {
                 }
                 elsif ( $slave eq $self->{info}{_info} ) {
                     if ( ! defined $j->{tables_info} ) {
-                        $j->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $j );
+                        $j->{tables_info} = $self->__get_tables_info( $dbh, $db, $schema, $data );
                     }
                     my $tbls_info = $self->__print_tables_info( $j );
                     # Choose
@@ -489,7 +474,7 @@ sub __print_join_statement {
     $join_stmt_pr =~ s/(?=\sLEFT\sOUTER\sJOIN)/\n\ /g;
     $join_stmt_pr .= "\n\n";
     my $line_fold = Text::LineFold->new( %{$self->{info}{line_fold}}, ColMax => ( term_size() )[0] - 2 );
-    print $self->{info}{clear_screen};
+    print CLEAR_SCREEN;
     print $line_fold->fold( '', ' ' x $self->{info}{stmt_init_tab}, $join_stmt_pr );
 }
 
