@@ -3,10 +3,10 @@ App::DBBrowser::Table;
 
 use warnings;
 use strict;
-use 5.008003;
+use 5.010000;
 no warnings 'utf8';
 
-our $VERSION = '0.049_09';
+our $VERSION = '0.041';
 
 use Clone                  qw( clone );
 use List::MoreUtils        qw( any first_index );
@@ -19,7 +19,9 @@ use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use App::DBBrowser::DB;
 #use App::DBBrowser::Table::Insert;  # "require"-d
-use App::DBBrowser::Auxil;
+use App::DBBrowser::Util;
+
+sub CLEAR_SCREEN () { "\e[H\e[J" }
 
 
 
@@ -31,7 +33,7 @@ sub new {
 
 sub __on_table {
     my ( $self, $sql, $dbh, $table, $select_from_stmt, $qt_columns, $pr_columns ) = @_;
-    my $auxil = App::DBBrowser::Auxil->new( $self->{info}, $self->{opt} );
+    my $util = App::DBBrowser::Util->new( $self->{info}, $self->{opt} );
     my $stmt_h = Term::Choose->new( $self->{info}{lyt_stmt_h} );
     my $sub_stmts = {
         Select => [ qw( print_table columns aggregate distinct where group_by having order_by limit lock ) ],
@@ -39,12 +41,7 @@ sub __on_table {
         Update => [ qw( commit set where ) ],
         Insert => [ qw( commit insert    ) ],
     };
-    my %map_sql_types = (
-        Insert => "INSERT INTO",
-        Update => "UPDATE",
-        Delete => "DELETE",
-    );
-    my $sql_types = [ 'Insert', 'Update', 'Delete' ];
+    my $sql_types = [ 'Delete', 'Update', 'Insert' ];
     my $sql_type = 'Select';
     my $lk = [ '  Lk0', '  Lk1' ];
     my %customize = (
@@ -65,39 +62,23 @@ sub __on_table {
     );
     my ( $DISTINCT, $ALL, $ASC, $DESC, $AND, $OR ) = ( "DISTINCT", "ALL", "ASC", "DESC", "AND", "OR" );
     if ( $self->{info}{lock} == 0 ) {
-        $auxil->__reset_sql( $sql );
+        $util->__reset_sql( $sql );
     }
     my $backup_sql = clone( $sql );
     my $old_idx = 1;
 
     CUSTOMIZE: while ( 1 ) {
         $backup_sql = clone( $sql ) if $sql_type eq 'Select';
-        $auxil->__print_sql_statement( $sql, $table, $sql_type );
+        $util->__print_sql_statement( $sql, $table, $sql_type );
         my $choices = [ $customize{hidden}, undef, @customize{@{$sub_stmts->{$sql_type}}} ];
         # Choose
         my $idx = $stmt_h->choose(
             $choices,
-            { %{$self->{info}{lyt_stmt_v}}, prompt => '', index => 1, default => $old_idx,
-            undef => $sql_type ne 'Select' ? $self->{info}{_back} : $self->{info}{back} }
+            { %{$self->{info}{lyt_stmt_v}}, prompt => '', index => 1, default => $old_idx, undef => $self->{info}{back} }
         );
-        if ( ! defined $idx || ! defined $choices->[$idx] ) {
-            if ( $sql_type eq 'Select'  ) {
-                last CUSTOMIZE;
-            }
-            else {
-                if ( $sql->{print}{where_stmt} || $sql->{print}{set_stmt} ) {
-                    $auxil->__reset_sql( $sql );
-                    next CUSTOMIZE;
-                }
-                else {
-                    $sql_type = 'Select';
-                    $old_idx = 1;
-                    $sql = clone $backup_sql;
-                    next CUSTOMIZE;
-                }
-            }
-        }
+        last CUSTOMIZE if ! defined $idx;
         my $custom = $choices->[$idx];
+        last CUSTOMIZE if ! defined $custom;
         if ( $self->{opt}{menu_sql_memory} ) {
             if ( $old_idx == $idx ) {
                 $old_idx = 1;
@@ -111,7 +92,7 @@ sub __on_table {
             if ( $self->{info}{lock} == 1 ) {
                 $self->{info}{lock} = 0;
                 $customize{lock} = $lk->[0];
-                $auxil->__reset_sql( $sql );
+                $util->__reset_sql( $sql );
             }
             elsif ( $self->{info}{lock} == 0 )   {
                 $self->{info}{lock} = 1;
@@ -119,13 +100,14 @@ sub __on_table {
             }
         }
         elsif ( $custom eq $customize{'insert'} ) {
+
             require App::DBBrowser::Table::Insert;
             my $tbl_in = App::DBBrowser::Table::Insert->new( $self->{info}, $self->{opt} );
-            $tbl_in->__insert_into( $sql, $table,$qt_columns, $pr_columns );
+            $tbl_in->__insert_into( $sql, $table,$qt_columns, $pr_columns, $backup_sql );
         }
         elsif ( $custom eq $customize{'columns'} ) {
             if ( ! ( $sql->{select_type} eq '*' || $sql->{select_type} eq 'chosen_cols' ) ) {
-                $auxil->__reset_sql( $sql );
+                $util->__reset_sql( $sql );
             }
             my @cols = ( @$pr_columns );
             $sql->{quote}{chosen_cols} = [];
@@ -136,7 +118,7 @@ sub __on_table {
                 my @pre = ( $self->{info}{ok} );
                 unshift @pre, undef if $self->{opt}{sssc_mode};
                 my $choices = [ @pre, @cols ];
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my @print_col = $stmt_h->choose(
                     $choices,
@@ -180,7 +162,7 @@ sub __on_table {
             DISTINCT: while ( 1 ) {
                 my $choices = [ $self->{info}{ok}, $DISTINCT, $ALL ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $select_distinct = $stmt_h->choose(
                     $choices
@@ -205,7 +187,7 @@ sub __on_table {
         }
         elsif ( $custom eq $customize{'aggregate'} ) {
             if ( $sql->{select_type} eq '*' || $sql->{select_type} eq 'chosen_cols' ) {
-                $auxil->__reset_sql( $sql );
+                $util->__reset_sql( $sql );
             }
             my @cols = ( @$pr_columns );
             $sql->{quote}{aggr_cols} = [];
@@ -215,7 +197,7 @@ sub __on_table {
             AGGREGATE: while ( 1 ) {
                 my $choices = [ $self->{info}{ok}, @{$self->{info}{avail_aggregate}} ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $aggr = $stmt_h->choose(
                     $choices
@@ -251,7 +233,7 @@ sub __on_table {
                     if ( $aggr eq 'COUNT' ) {
                         my $choices = [ $ALL, $DISTINCT ];
                         unshift @$choices, undef if $self->{opt}{sssc_mode};
-                        $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                        $util->__print_sql_statement( $sql, $table, $sql_type );
                         # Choose
                         my $all_or_distinct = $stmt_h->choose(
                             $choices
@@ -268,7 +250,7 @@ sub __on_table {
                     }
                     my $choices = [ @cols ];
                     unshift @$choices, undef if $self->{opt}{sssc_mode};
-                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                    $util->__print_sql_statement( $sql, $table, $sql_type );
                     # Choose
                     my $print_col = $stmt_h->choose(
                         $choices
@@ -301,12 +283,13 @@ sub __on_table {
                 my @pre = ( $self->{info}{ok} );
                 unshift @pre, undef if $self->{opt}{sssc_mode};
                 my $choices = [ @pre, @cols ];
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
-                my $print_col = $stmt_h->choose(
+                my @print_col = $stmt_h->choose(
                     $choices,
+                    { no_spacebar => [ 0 .. $#pre ] }
                 );
-                if ( ! defined $print_col ) {
+                if ( ! @print_col || ! defined $print_col[0] ) {
                     if ( @{$sql->{quote}{set_args}} ) {
                         $sql->{quote}{set_args} = [];
                         $sql->{quote}{set_stmt} = " SET";
@@ -320,7 +303,26 @@ sub __on_table {
                         last SET;
                     }
                 }
-                if ( $print_col eq $self->{info}{ok} ) {
+                if ( $print_col[0] eq $self->{info}{ok} ) {
+                    shift @print_col;
+                    for my $print_col ( @print_col ) {
+                        ( my $quote_col = $qt_columns->{$print_col} ) =~ s/\sAS\s\S+\z//;
+                        $sql->{quote}{set_stmt} .= $col_sep . $quote_col . ' =';
+                        $sql->{print}{set_stmt} .= $col_sep . $print_col . ' =';
+                        $util->__print_sql_statement( $sql, $table, $sql_type );
+                        # Readline
+                        my $value = $trs->readline( $print_col . ': ' );
+                        if ( ! defined $value ) {
+                            $sql->{quote}{set_args} = [];
+                            $sql->{quote}{set_stmt} = '';
+                            $sql->{print}{set_stmt} = '';
+                            return;
+                        }
+                        $sql->{quote}{set_stmt} .= ' ' . '?';
+                        $sql->{print}{set_stmt} .= ' ' . "'$value'";
+                        push @{$sql->{quote}{set_args}}, $value;
+                        $col_sep = ', ';
+                    }
                     if ( $col_sep eq ' ' ) {
                         $sql->{quote}{set_stmt} = '';
                         $sql->{print}{set_stmt} = '';
@@ -328,29 +330,24 @@ sub __on_table {
                     }
                     last SET;
                 }
-                ( my $quote_col = $qt_columns->{$print_col} ) =~ s/\sAS\s\S+\z//;
-                $sql->{quote}{set_stmt} .= $col_sep . $quote_col . ' =';
-                $sql->{print}{set_stmt} .= $col_sep . $print_col . ' =';
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
-                # Readline
-                my $value = $trs->readline( $print_col . ': ' );
-                if ( ! defined $value ) {
-                    if ( @{$sql->{quote}{set_args}} ) {
+                for my $print_col ( @print_col ) {
+                    ( my $quote_col = $qt_columns->{$print_col} ) =~ s/\sAS\s\S+\z//;
+                    $sql->{quote}{set_stmt} .= $col_sep . $quote_col . ' =';
+                    $sql->{print}{set_stmt} .= $col_sep . $print_col . ' =';
+                    $util->__print_sql_statement( $sql, $table, $sql_type );
+                    # Readline
+                    my $value = $trs->readline( $print_col . ': ' );
+                    if ( ! defined $value ) {
                         $sql->{quote}{set_args} = [];
-                        $sql->{quote}{set_stmt} = " SET";
-                        $sql->{print}{set_stmt} = " SET";
-                        $col_sep = ' ';
-                        next SET;
-                    }
-                    else {
-                        $sql = clone( $backup_sql );
-                        last SET;
+                        $sql->{quote}{set_stmt} = '';
+                        $sql->{print}{set_stmt} = '';
+                        return;
                     }
+                    $sql->{quote}{set_stmt} .= ' ' . '?';
+                    $sql->{print}{set_stmt} .= ' ' . "'$value'";
+                    push @{$sql->{quote}{set_args}}, $value;
+                    $col_sep = ', ';
                 }
-                $sql->{quote}{set_stmt} .= ' ' . '?';
-                $sql->{print}{set_stmt} .= ' ' . "'$value'";
-                push @{$sql->{quote}{set_args}}, $value;
-                $col_sep = ', ';
             }
         }
         elsif ( $custom eq $customize{'where'} ) {
@@ -372,7 +369,7 @@ sub __on_table {
                 elsif ( $self->{opt}{parentheses_w} == 2 ) {
                     push @choices, $unclosed ? ')' : '(';
                 }
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $print_col = $stmt_h->choose(
                     [ @pre, @choices ]
@@ -407,7 +404,7 @@ sub __on_table {
                 if ( $count > 0 && $sql->{quote}{where_stmt} !~ /\(\z/ ) {
                     my $choices = [ $AND, $OR ];
                     unshift @$choices, undef if $self->{opt}{sssc_mode};
-                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                    $util->__print_sql_statement( $sql, $table, $sql_type );
                     # Choose
                     $AND_OR = $stmt_h->choose(
                         $choices
@@ -447,7 +444,7 @@ sub __on_table {
         }
         elsif ( $custom eq $customize{'group_by'} ) {
             if ( $sql->{select_type} eq '*' || $sql->{select_type} eq 'chosen_cols' ) {
-                $auxil->__reset_sql( $sql );
+                $util->__reset_sql( $sql );
             }
             my @cols = ( @$pr_columns );
             my $col_sep = ' ';
@@ -461,7 +458,7 @@ sub __on_table {
                 my @pre = ( $self->{info}{ok} );
                 unshift @pre, undef if $self->{opt}{sssc_mode};
                 my $choices = [ @pre, @cols ];
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my @print_col = $stmt_h->choose(
                     $choices,
@@ -531,7 +528,7 @@ sub __on_table {
                 elsif ( $self->{opt}{parentheses_h} == 2 ) {
                     push @choices, $unclosed ? ')' : '(';
                 }
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $aggr = $stmt_h->choose(
                     [ @pre, @choices ]
@@ -566,7 +563,7 @@ sub __on_table {
                 if ( $count > 0 && $sql->{quote}{having_stmt} !~ /\(\z/ ) {
                     my $choices = [ $AND, $OR ];
                     unshift @$choices, undef if $self->{opt}{sssc_mode};
-                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                    $util->__print_sql_statement( $sql, $table, $sql_type );
                     # Choose
                     $AND_OR = $stmt_h->choose(
                         $choices
@@ -613,7 +610,7 @@ sub __on_table {
                     $print_aggr                 =           $aggr . "(";
                     my $choices = [ @cols ];
                     unshift @$choices, undef if $self->{opt}{sssc_mode};
-                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                    $util->__print_sql_statement( $sql, $table, $sql_type );
                     # Choose
                     $print_col = $stmt_h->choose(
                         $choices
@@ -659,7 +656,7 @@ sub __on_table {
             ORDER_BY: while ( 1 ) {
                 my $choices = [ $self->{info}{ok}, @cols ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $print_col = $stmt_h->choose(
                     $choices
@@ -689,7 +686,7 @@ sub __on_table {
                 $sql->{print}{order_by_stmt} .= $col_sep . $print_col;
                 $choices = [ $ASC, $DESC ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $direction = $stmt_h->choose(
                     $choices
@@ -716,7 +713,7 @@ sub __on_table {
             LIMIT: while ( 1 ) {
                 my $choices = [ $self->{info}{ok}, $only_limit, $offset_and_limit ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $choice = $stmt_h->choose(
                     $choices
@@ -769,7 +766,6 @@ sub __on_table {
             if ( $sql_type eq 'Insert' ) {
                 my $obj_opt = App::DBBrowser::Opt->new( $self->{info}, $self->{opt} );
                 $obj_opt->__config_insert( 0 );
-                $sql = clone( $backup_sql ); ###
                 next CUSTOMIZE;
             }
             my @functions = @{$self->{info}{keys_hidd_func_pr}};
@@ -811,7 +807,7 @@ sub __on_table {
                     $default = 1;
                 }
                 my $choices = [ @pre, map( "- $_", @cols ) ];
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $i = $stmt_h->choose(
                     $choices,
@@ -828,16 +824,17 @@ sub __on_table {
                 }
                 if ( $print_col eq $choose_type ) {
                     my $choices = [ undef, map( "- $_", @$sql_types ) ];
+                    $util->__print_sql_statement( $sql, $table, $sql_type );
                     # Choose
                     my $st = $stmt_h->choose(
                         $choices,
-                        { %{$self->{info}{lyt_stmt_v}}, prompt => 'Choose SQL type:', default => 0, clear_screen => 1 }
+                        { %{$self->{info}{lyt_stmt_v}}, prompt => '', default => 0 }
                     );
                     if ( defined $st ) {
                         $st =~ s/^-\ //;
                         $sql_type = $st;
                         $old_idx = 1;
-                        $auxil->__reset_sql( $sql );
+                        $util->__reset_sql( $sql );
                     }
                     last HIDDEN;
                 }
@@ -870,7 +867,7 @@ sub __on_table {
                     $changed++;
                     next HIDDEN;
                 }
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my $function = $stmt_h->choose(
                     [ undef, map( "  $_", @functions ) ],
@@ -881,9 +878,9 @@ sub __on_table {
                 }
                 $function =~ s/^\s\s//;
                 ( my $quote_col = $qt_columns->{$print_col} ) =~ s/\sAS\s\S+\z//;
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-                my ( $quote_hidd, $print_hidd ) = $self->__col_functions( $function, $quote_col, $print_col );
+                my ( $quote_hidd, $print_hidd ) = $obj_db->col_functions( $function, $quote_col, $print_col );
                 if ( ! defined $quote_hidd ) {
                     next HIDDEN;
                 }
@@ -930,21 +927,27 @@ sub __on_table {
                 push @arguments, $self->{opt}{max_rows};
             }
             local $| = 1;
-            print $self->{info}{clear_screen};
-            print 'Database : ...' . "\n" if $self->{opt}{progress_bar};
+            print CLEAR_SCREEN;
+            say 'Database : ...' if $self->{opt}{progress_bar};
             my $sth = $dbh->prepare( $select );
             $sth->execute( @arguments );
             my $col_names = $sth->{NAME};
             my $all_arrayref = $sth->fetchall_arrayref;
             unshift @$all_arrayref, $col_names;
-            print $self->{info}{clear_screen};
+            print CLEAR_SCREEN;
             return $all_arrayref;
         }
         elsif ( $custom eq $customize{'commit'} ) {
             my ( $qt_table ) = $select_from_stmt =~ /^SELECT\s.*?\sFROM\s(.*)\z/;
+            my %map_sql_types = (
+                Delete => "DELETE",
+                Update => "UPDATE",
+                Insert => "INSERT INTO",
+            );
+
             local $| = 1;
-            print $self->{info}{clear_screen};
-            print 'Database : ...' . "\n" if $self->{opt}{progress_bar};
+            print CLEAR_SCREEN;
+            say 'Database : ...' if $self->{opt}{progress_bar};
             my $stmt = $map_sql_types{$sql_type};
             my $to_execute;
             my $transaction;
@@ -982,25 +985,24 @@ sub __on_table {
                     }
                     my $nr_rows   = $sql_type eq 'Insert' ? @$to_execute : $sth->rows;
                     my $commit_ok = sprintf $commit_fmt{$sql_type}, $nr_rows;
-                    $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                    $util->__print_sql_statement( $sql, $table, $sql_type );
                     my $choices = [ undef,  $commit_ok ];
                     # Choose
                     my $choice = $stmt_h->choose(
                         $choices,
-                        { %{$self->{info}{lyt_stmt_v}}, undef => '  BACK', prompt => 'Choose:' }
+                        { %{$self->{info}{lyt_stmt_v}}, undef => '  BACK', prompt => '' }
                     );
                     if ( defined $choice && $choice eq $commit_ok ) {;
                         $dbh->commit;
                     }
                     else {
                         $dbh->rollback;
-                        $auxil->__reset_sql( $sql );
-                        next CUSTOMIZE;
                     }
                     1;
                     }
                 ) {
-                    $auxil->__print_error_message( "$@rolling back ...\n", 'Commit' );
+                    say 'Commit:';
+                    $util->__print_error_message( "$@rolling back ...\n" );
                     eval { $dbh->rollback };#
                 }
             }
@@ -1014,7 +1016,7 @@ sub __on_table {
                     ( $nr_rows ) = $dbh->selectrow_array( $count_stmt, undef, @{$sql->{quote}{where_args}} );
                 }
                 my $commit_ok = sprintf $commit_fmt{$sql_type}, $nr_rows;
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 my $choices = [ undef,  $commit_ok ];
                 # Choose
                 my $choice = $stmt_h->choose(
@@ -1027,11 +1029,8 @@ sub __on_table {
                         $sth->execute( @$values );
                     }
                 }
-                else {
-                    $auxil->__reset_sql( $sql );
-                    next CUSTOMIZE;
-                }
             }
+            $util->__reset_sql( $sql );
             $sql_type = 'Select';
             $old_idx = 1;
             $sql = clone $backup_sql;
@@ -1057,7 +1056,7 @@ sub __unambiguous_key {
 
 sub __set_operator_sql {
     my ( $self, $sql, $clause, $table, $cols, $qt_columns, $quote_col, $sql_type ) = @_;
-    my $auxil = App::DBBrowser::Auxil->new( $self->{info}, $self->{opt} );
+    my $util = App::DBBrowser::Util->new( $self->{info}, $self->{opt} );
     my ( $stmt, $args );
     my $stmt_h = Term::Choose->new( $self->{info}{lyt_stmt_h} );
     if ( $clause eq 'where' ) {
@@ -1070,7 +1069,7 @@ sub __set_operator_sql {
     }
     my $choices = [ @{$self->{opt}{operators}} ];
     unshift @$choices, undef if $self->{opt}{sssc_mode};
-    $auxil->__print_sql_statement( $sql, $table, $sql_type );
+    $util->__print_sql_statement( $sql, $table, $sql_type );
     # Choose
     my $operator = $stmt_h->choose(
         $choices
@@ -1097,7 +1096,7 @@ sub __set_operator_sql {
             $sql->{print}{$stmt} .= '(';
 
             IN: while ( 1 ) {
-                $auxil->__print_sql_statement( $sql, $table, $sql_type );
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Readline
                 my $value = $trs->readline( 'Value: ' );
                 if ( ! defined $value ) {
@@ -1124,7 +1123,7 @@ sub __set_operator_sql {
             }
         }
         elsif ( $operator =~ /^(?:NOT\s)?BETWEEN\z/ ) {
-            $auxil->__print_sql_statement( $sql, $table, $sql_type );
+            $util->__print_sql_statement( $sql, $table, $sql_type );
             # Readline
             my $value_1 = $trs->readline( 'Value: ' );
             if ( ! defined $value_1 ) {
@@ -1136,7 +1135,7 @@ sub __set_operator_sql {
             $sql->{quote}{$stmt} .= ' ' . '?' .      ' AND';
             $sql->{print}{$stmt} .= ' ' . $value_1 . ' AND';
             push @{$sql->{quote}{$args}}, $value_1;
-            $auxil->__print_sql_statement( $sql, $table, $sql_type );
+            $util->__print_sql_statement( $sql, $table, $sql_type );
             # Readline
             my $value_2 = $trs->readline( 'Value: ' );
             if ( ! defined $value_2 ) {
@@ -1151,7 +1150,7 @@ sub __set_operator_sql {
         }
         elsif ( $operator =~ /REGEXP\z/ ) {
             $sql->{print}{$stmt} .= ' ' . $operator;
-            $auxil->__print_sql_statement( $sql, $table, $sql_type );
+            $util->__print_sql_statement( $sql, $table, $sql_type );
             # Readline
             my $value = $trs->readline( 'Pattern: ' );
             if ( ! defined $value ) {
@@ -1167,12 +1166,12 @@ sub __set_operator_sql {
             }
             $sql->{quote}{$stmt} =~ s/.*\K\s\Q$quote_col\E//;
             my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-            $sql->{quote}{$stmt} .= $obj_db->sql_regexp( $quote_col, $operator =~ /^NOT/ ? 1 : 0, $self->{opt}{regexp_case} );
+            $sql->{quote}{$stmt} .= $obj_db->sql_regexp( $quote_col, $operator =~ /^NOT/ ? 1 : 0 );
             $sql->{print}{$stmt} .= ' ' . "'$value'";
             push @{$sql->{quote}{$args}}, $value;
         }
         else {
-            $auxil->__print_sql_statement( $sql, $table, $sql_type );
+            $util->__print_sql_statement( $sql, $table, $sql_type );
             my $prompt = $operator =~ /LIKE\z/ ? 'Pattern: ' : 'Value: ';
             # Readline
             my $value = $trs->readline( $prompt );
@@ -1198,7 +1197,7 @@ sub __set_operator_sql {
         $sql->{print}{$stmt} .= ' ' . $operator;
         my $choices = [ @$cols ];
         unshift @$choices, undef if $self->{opt}{sssc_mode};
-        $auxil->__print_sql_statement( $sql, $table, $sql_type );
+        $util->__print_sql_statement( $sql, $table, $sql_type );
         # Choose
         my $print_col = $stmt_h->choose(
             $choices,
@@ -1235,55 +1234,6 @@ sub __set_operator_sql {
 }
 
 
-sub __col_functions {
-    my ( $self, $func, $quote_col, $print_col ) = @_;
-    my $obj_db = App::DBBrowser::DB->new( $self->{info}, $self->{opt} );
-    my $obj_ch = Term::Choose->new();
-    my ( $quote_f, $print_f );
-    $print_f = $self->{info}{hidd_func_pr}{$func} . '(' . $print_col . ')';
-    if ( $func =~ /^Epoch_to_Date(?:Time)?\z/ ) {
-        my $prompt = "$print_f\nInterval:";
-        my ( $microseconds, $milliseconds, $seconds ) = (
-            '  ****************   Micro-Second',
-            '  *************      Milli-Second',
-            '  **********               Second' );
-        my $choices = [ undef, $microseconds, $milliseconds, $seconds ];
-        # Choose
-        my $interval = $obj_ch->choose(
-            $choices,
-            { %{$self->{info}{lyt_stmt_v}}, prompt => $prompt }
-        );
-        return if ! defined $interval;
-        my $div = $interval eq $microseconds ? 1000000 :
-                  $interval eq $milliseconds ? 1000 : 1;
-        if ( $func eq 'Epoch_to_DateTime' ) {
-            $quote_f = $obj_db->epoch_to_datetime( $quote_col, $div ); #
-        }
-        else {
-            $quote_f = $obj_db->epoch_to_date( $quote_col, $div ); #
-        }
-    }
-    elsif ( $func eq 'Truncate' ) {
-        my $prompt = "TRUNC $print_col\nDecimal places:";
-        my $choices = [ undef, 0 .. 9 ];
-        # Choose
-        my $precision = $obj_ch->choose(
-            $choices,
-            { %{$self->{info}{lyt_stmt_h}}, prompt => $prompt }
-        );
-        return if ! defined $precision;
-        $quote_f = $obj_db->truncate( $quote_col, $precision );
-    }
-    elsif ( $func eq 'Bit_Length' ) {
-        $quote_f = $obj_db->bit_length( $quote_col );
-    }
-    elsif ( $func eq 'Char_Length' ) {
-        $quote_f = $obj_db->char_length( $quote_col );
-    }
-    return $quote_f, $print_f;
-}
-
-
 
 
 1;
