@@ -3,9 +3,9 @@ App::DBBrowser::Util;
 
 use warnings;
 use strict;
-use 5.008003;
+use 5.010000;
 
-our $VERSION = '0.049_05';
+our $VERSION = '0.041';
 
 use Term::Choose           qw( choose );
 use Term::Choose::Util     qw( term_size );
@@ -76,10 +76,8 @@ sub __print_sql_statement {
 
 
 sub __print_error_message {
-    my ( $self, $message, $title ) = @_;
-    print "$title:\n" if $title;
+    my ( $self, $message ) = @_;
     utf8::decode( $message );
-    $message =~ s/\sat(?:(?!\sat\s).)+// if ! $self->{opt}{debug};
     print $message;
     choose(
         [ 'Press ENTER to continue' ],
